import * as fs from 'node:fs/promises';
import * as path from 'node:path';
import * as fs$1 from 'node:fs';
import { XMLBuilder, XMLParser } from 'fast-xml-parser';
import { createHash } from 'node:crypto';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

const XML_HEADER = '<?xml version="1.0" encoding="UTF-8"?>';
const INDENT = "    ";

function buildReassembledFile(combinedXmlContents, filePath, xmlElement, xmlNamespace) {
    return __awaiter(this, void 0, void 0, function* () {
        let finalXmlContent = combinedXmlContents.join("\n");
        finalXmlContent = finalXmlContent.replace(/<\?xml version="1.0" encoding="UTF-8"\?>/g, "");
        finalXmlContent = finalXmlContent.replace(`<${xmlElement}>`, "");
        finalXmlContent = finalXmlContent.replace(`</${xmlElement}>`, "");
        finalXmlContent = finalXmlContent.replace(/<!\[CDATA\[\s*([\s\S]*?)\s*]]>/g, (match, cdataContent) => {
            const trimmedContent = cdataContent.trim();
            const lines = trimmedContent.split("\n");
            const indentedLines = lines.map((line) => line.replace(/^\s*/, ""));
            return `<![CDATA[\n${INDENT}${indentedLines.join(`\n${INDENT}`)}\n]]>`;
        });
        finalXmlContent = finalXmlContent.replace(/(\n\s*){2,}/g, `\n${INDENT}`);
        const openTag = xmlNamespace
            ? `<${xmlElement} xmlns="${xmlNamespace}">`
            : `<${xmlElement}>`;
        const closeTag = `</${xmlElement}>`;
        yield fs.writeFile(filePath, `${XML_HEADER}\n${openTag}${finalXmlContent}${closeTag}`);
        console.log(`Created reassembled file: ${filePath}`);
    });
}

class ReassembleXMLFileHandler {
    processFilesInDirectory(dirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const combinedXmlContents = [];
            const files = yield fs.readdir(dirPath);
            files.sort((fileA, fileB) => {
                const fullNameA = fileA.split(".")[0].toLowerCase();
                const fullNameB = fileB.split(".")[0].toLowerCase();
                return fullNameA.localeCompare(fullNameB);
            });
            for (const file of files) {
                const filePath = path.join(dirPath, file);
                const fileStat = yield fs.stat(filePath);
                if (fileStat.isFile()) {
                    const xmlContent = yield fs.readFile(filePath, "utf-8");
                    combinedXmlContents.push(xmlContent);
                }
                else if (fileStat.isDirectory()) {
                    const subdirectoryContents = yield this.processFilesInDirectory(filePath);
                    combinedXmlContents.push(...subdirectoryContents);
                }
            }
            return combinedXmlContents;
        });
    }
    reassemble(xmlAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xmlElement, xmlPath, xmlNamespace, fileExtension } = xmlAttributes;
            const combinedXmlContents = [];
            const filesInxmlPath = yield fs.readdir(xmlPath);
            for (const file of filesInxmlPath) {
                const filePath = path.join(xmlPath, file);
                const fileStat = yield fs.stat(filePath);
                if (fileStat.isFile()) {
                    const xmlContent = yield fs.readFile(filePath, "utf-8");
                    combinedXmlContents.push(xmlContent);
                }
                else if (fileStat.isDirectory()) {
                    const subdirectoryContents = yield this.processFilesInDirectory(filePath);
                    combinedXmlContents.push(...subdirectoryContents);
                }
            }
            const parentDirectory = path.dirname(xmlPath);
            const subdirectoryBasename = path.basename(xmlPath);
            const fileName = fileExtension
                ? `${subdirectoryBasename}.${fileExtension}`
                : `${subdirectoryBasename}.xml`;
            const filePath = path.join(parentDirectory, fileName);
            yield buildReassembledFile(combinedXmlContents, filePath, xmlElement, xmlNamespace);
        });
    }
}

const XML_PARSER_OPTION = {
    commentPropName: "!---",
    ignoreAttributes: false,
    ignoreNameSpace: false,
    parseTagValue: false,
    parseNodeValue: false,
    parseAttributeValue: false,
    trimValues: true,
    processEntities: false,
    cdataPropName: "![CDATA[",
};
const JSON_PARSER_OPTION = Object.assign(Object.assign({}, XML_PARSER_OPTION), { format: true, indentBy: INDENT, suppressBooleanAttributes: false, suppressEmptyNode: false });

function findUniqueIdElement(element, uniqueIdElements) {
    if (uniqueIdElements === undefined) {
        return getShortHash(element);
    }
    const uniqueIdElementsArray = uniqueIdElements.split(",");
    for (const fieldName of uniqueIdElementsArray) {
        if (element[fieldName] !== undefined) {
            if (typeof element[fieldName] === "string") {
                return element[fieldName];
            }
        }
    }
    for (const key in element) {
        if (typeof element[key] === "object" && element[key] !== null) {
            const childFieldName = findUniqueIdElement(element[key], uniqueIdElements);
            if (childFieldName !== undefined) {
                return childFieldName;
            }
        }
    }
    return getShortHash(element);
}
function getShortHash(element) {
    const hash = createHash("sha256");
    hash.update(JSON.stringify(element));
    const fullHash = hash.digest("hex");
    return fullHash.slice(0, 8);
}

function buildNestedElements(element, indentLevel = 2) {
    const xmlBuilder = new XMLBuilder(JSON_PARSER_OPTION);
    const xmlString = xmlBuilder.build(element);
    const formattedXml = xmlString
        .split("\n")
        .map((line) => `${" ".repeat(indentLevel * INDENT.length)}${line}`)
        .join("\n")
        .trimEnd();
    return formattedXml;
}

function buildDisassembledFiles(xmlString, metadataPath, uniqueIdElements, xmlElement, baseName, indent) {
    const xmlParser = new XMLParser(XML_PARSER_OPTION);
    const result = xmlParser.parse(xmlString);
    const rootElementName = Object.keys(result)[1];
    const rootElement = result[rootElementName];
    let leafContent = "";
    let leafCount = 0;
    Object.keys(rootElement)
        .filter((key) => !key.startsWith("@"))
        .forEach((key) => {
        if (Array.isArray(rootElement[key])) {
            for (const element of rootElement[key]) {
                if (typeof element === "object") {
                    buildNestedFile(element, metadataPath, uniqueIdElements, key, indent);
                }
                else {
                    const fieldValue = element;
                    leafContent += `${indent}<${key}>${String(fieldValue)}</${key}>\n`;
                    leafCount++;
                }
            }
        }
        else if (typeof rootElement[key] === "object") {
            buildNestedFile(rootElement[key], metadataPath, uniqueIdElements, key, indent);
        }
        else {
            const fieldValue = rootElement[key];
            leafContent += `${indent}<${key}>${String(fieldValue)}</${key}>\n`;
            leafCount++;
        }
    });
    if (leafCount > 0) {
        let leafFile = `${XML_HEADER}\n`;
        leafFile += `<${xmlElement}>\n`;
        const sortedLeafContent = leafContent
            .split("\n")
            .filter((line) => line.trim() !== "")
            .sort()
            .join("\n");
        leafFile += sortedLeafContent;
        leafFile += `\n</${xmlElement}>`;
        const leafOutputPath = path.join(metadataPath, `${baseName}.xml`);
        fs$1.writeFileSync(leafOutputPath, leafFile);
        console.log(`All leaf elements saved to: ${leafOutputPath}`);
    }
}
function buildNestedFile(element, metadataPath, uniqueIdElements, parentKey, indent) {
    let elementContent = "";
    elementContent += `${XML_HEADER}\n`;
    const fieldName = findUniqueIdElement(element, uniqueIdElements);
    const outputDirectory = path.join(metadataPath, parentKey);
    const outputFileName = `${fieldName}.${parentKey}-meta.xml`;
    const outputPath = path.join(outputDirectory, outputFileName);
    fs$1.mkdirSync(outputDirectory, { recursive: true });
    elementContent = buildNestedElements(element);
    let decomposeFileContents = `${XML_HEADER}\n`;
    decomposeFileContents += `${indent}<${parentKey}>\n`;
    decomposeFileContents += `${elementContent}\n`;
    decomposeFileContents += `${indent}</${parentKey}>\n`;
    fs$1.writeFileSync(outputPath, decomposeFileContents);
    console.log(`XML content saved to: ${outputPath}`);
}

class DisassembleXMLFileHandler {
    disassemble(xmlAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xmlPath, xmlElement, uniqueIdElements } = xmlAttributes;
            const files = yield fs.readdir(xmlPath);
            for (const file of files) {
                const filePath = path.join(xmlPath, file);
                yield this.processFile({
                    xmlPath,
                    filePath,
                    uniqueIdElements,
                    xmlElement,
                });
            }
        });
    }
    processFile(xmlAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xmlPath, filePath, xmlElement, uniqueIdElements } = xmlAttributes;
            if (filePath.endsWith(".xml")) {
                console.log(`Parsing file: ${filePath}`);
                const xmlContent = yield fs.readFile(filePath, "utf-8");
                const baseName = path
                    .basename(filePath, path.extname(filePath))
                    .split(".")[0];
                let outputPath;
                outputPath = path.join(xmlPath, baseName);
                buildDisassembledFiles(xmlContent, outputPath, uniqueIdElements, xmlElement, baseName, INDENT);
            }
        });
    }
}

export { DisassembleXMLFileHandler, ReassembleXMLFileHandler };
//# sourceMappingURL=index.mjs.map
